// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "EndlessVendetta/EndlessVendettaCharacter.h"
#include "EndlessVendetta/MainSaveGameClass.h"
#include "EndlessVendetta/BountySystem/MainBountyClass.h"
#include "BountyHunterCharacter.generated.h"

/**
 * 
 */
UCLASS()
class ENDLESSVENDETTA_API ABountyHunterCharacter : public AEndlessVendettaCharacter
{
	GENERATED_BODY()
// ------------------- ATTRIBUTES ------------------------------

	// Array of Main Bounties' Classes in Order
	UPROPERTY(EditDefaultsOnly, Category = "Bounty Hunter")
	TArray<TSubclassOf<AMainBountyClass>> MainBountyClasses;
	
	// Reference to the Currently Active Main Bounty
	AMainBountyClass* CurrentMainBounty;

	// Index of Currently Active Main Bounty, Used for MainBountyClasses
	int CurrentMainBountyIndex = 0;

	// Name of open world, so that the correct bounties can be spawned based on level
	UPROPERTY(EditDefaultsOnly, Category = "Bounty Hunter")
	FString OpenWorldLevelName = "ControlTutorialLevel";
	
protected:

	
public:

// ------------------- METHODS ---------------------------------
private:
	// Spawns Current Main Bounty along with its Side Bounties, and spawns its appropriate CP's based on level
	void SpawnMainBounty();
	
	// Collects Reward for Current Main Bounty and Increments the Main Bounty Index
	void CompleteCurrentMainBounty();

	// Replaces Normal Bounty Info UI with Unique Game Over UI
	void AllBountiesCompleted();
	
protected:
	// Called When Player Spawns
	virtual void BeginPlay() override;

	// Called every frame
	virtual void Tick(float DeltaTime) override;
public:
	// Used by Final Checkpoint to always load the Level set as the Open Level in here
	FString GetOpenWorldLevelName()
	{
		return OpenWorldLevelName;
	}
	
};
